/** -*- c++ -*- **************************************************************
 **
 ** Copyright 2014 Daniel Paschka (daniel@drachenschuppe.de)
 **
 ** Licensed under the Apache License, Version 2.0 (the "License");
 ** you may not use this file except in compliance with the License.
 ** You may obtain a copy of the License at
 **
 ** http://www.apache.org/licenses/LICENSE-2.0
 **
 ** Unless required by applicable law or agreed to in writing, software
 ** distributed under the License is distributed on an "AS IS" BASIS,
 ** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ** See the License for the specific language governing permissions and
 ** limitations under the License.
 *****************************************************************************/

#ifndef _TARGETIF
#define _TARGETIF

#include <stdint.h>

class MCU_AVR32;

class targetif
{
	MCU_AVR32 *mcu;
	void dumpMemory(int len, unsigned char *data);
	int  hwpoint_set(int num, uint32_t *addrlist, uint32_t areg, uint32_t creg, uint32_t addr, int mode);
	int  hwpoint_remove(int num, uint32_t *addrlist, uint32_t creg, uint32_t addr);

	unsigned char flashPage[512]; // Buffer for single page
	int           lastFlashPage;
	uint32_t      dbgctrl;
	uint32_t      pc;

	uint32_t     *bp_addr;
	int           bpts;
	uint32_t     *wp_addr;
	int           wpts;

	public:
	static const char *memory_map;
	int last_stop_sig;
	int wait_stop;

	targetif(MCU_AVR32 *mcu);
	~targetif();
	int init();
	int stop();
	int cont();
	int step();
	int reset();
	int check_stop();
	int setDC(uint32_t val);
	int breakpoint_set(int type, uint32_t addr);
	int breakpoint_remove(int type, uint32_t addr);
	int hwpoint_remove_all();
	int read_system_register(int reg, uint32_t &val);
	int write_system_register(int reg, uint32_t val);
	int read_mcu_register(int reg, uint32_t &val);
	int write_mcu_register(int reg, uint32_t val);
	int memory_read(uint32_t addr, int len, unsigned char *data);
	int memory_write(uint32_t addr, int len, unsigned char *data);
	void flashPageFinish();
	int flashErase(uint32_t addr, int len);
	int flashWrite(uint32_t addr, int len, unsigned char *data);
	void flashFinish();
};

#endif // _TARGETIF
