/** -*- c++ -*- **************************************************************
 **
 ** Copyright 2014 Daniel Paschka (daniel@drachenschuppe.de)
 **
 ** Licensed under the Apache License, Version 2.0 (the "License");
 ** you may not use this file except in compliance with the License.
 ** You may obtain a copy of the License at
 **
 ** http://www.apache.org/licenses/LICENSE-2.0
 **
 ** Unless required by applicable law or agreed to in writing, software
 ** distributed under the License is distributed on an "AS IS" BASIS,
 ** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ** See the License for the specific language governing permissions and
 ** limitations under the License.
 *****************************************************************************/

#ifndef _TCPSTREAM_H
#define _TCPSTREAM_H

#ifdef __WIN32__
#	include <winsock2.h>
#	define socklen_t int
#else
#	include <sys/socket.h>
#	include <arpa/inet.h>
#	include <netinet/tcp.h>
#endif

#include <streambuf>
#include <iostream>

class tcpstreambuf : public std::streambuf
{
protected:
	struct sockaddr_in _serverAddr;
	struct sockaddr_in _clientAddr;
	int                _serverfd;
	int                _clientfd;
	char               _buffer[4096];
	const std::size_t  _putBackCnt;

	std::streamsize showmanyc();
	int underflow();
	std::streamsize xsputn (const char* s, std::streamsize n);

public:
	tcpstreambuf();
	void openListen(uint16_t port, int socketOption);
	struct sockaddr_in waitForConnect();
};


class tcpstream : public std::iostream
{
private:
	tcpstreambuf _buf;

public:
	tcpstream() : _buf()
	{
		this->init(&_buf);
	}

	void openListen(uint16_t port, int socketOption) { _buf.openListen(port, socketOption); }
	struct sockaddr_in waitForConnect() { return _buf.waitForConnect(); }
};

#endif // _TCPSTREAM_H

