/** -*- c++ -*- **************************************************************
 **
 ** Copyright 2014 Lynx Technik AG (www.lynx-technik.com)
 **
 ** Licensed under the Apache License, Version 2.0 (the "License");
 ** you may not use this file except in compliance with the License.
 ** You may obtain a copy of the License at
 **
 ** http://www.apache.org/licenses/LICENSE-2.0
 **
 ** Unless required by applicable law or agreed to in writing, software
 ** distributed under the License is distributed on an "AS IS" BASIS,
 ** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ** See the License for the specific language governing permissions and
 ** limitations under the License.
 *****************************************************************************/


#ifdef __WIN32__
#	include <winsock2.h>
#else
#	include <arpa/inet.h>
#endif // __WIN32__

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "elfread.h"
#include "elf_common.h"
#include "elf32.h"

struct page_st *ELFread(int fd)
{
	struct page_st *pages = new struct page_st[1024];
	for (int p=0; p<1024; p++)
	{
		pages[p].used = 0;
		memset(pages[p].data, 0xff, PAGESIZE);
	}

	Elf32_Ehdr ehdr;

	// fetch, parse and store ELF header
	read(fd, &ehdr, sizeof(ehdr));
	ehdr.e_phoff = htonl(ehdr.e_phoff);
	ehdr.e_phnum = htons(ehdr.e_phnum);

	lseek(fd, ehdr.e_phoff, 0);
	// printf("num phdrs: %x\n", ehdr.e_phnum);

	Elf32_Phdr *phdr = new Elf32_Phdr[ehdr.e_phnum];
	for (int ph=0; ph<ehdr.e_phnum; ph++)
	{
		read(fd, &phdr[ph], sizeof(Elf32_Phdr));
		phdr[ph].p_type   = htonl(phdr[ph].p_type);
		phdr[ph].p_flags  = htonl(phdr[ph].p_flags);
		phdr[ph].p_offset = htonl(phdr[ph].p_offset);
		phdr[ph].p_filesz = htonl(phdr[ph].p_filesz);
		phdr[ph].p_paddr  = htonl(phdr[ph].p_paddr);
	}

	for (int ph=0; ph<ehdr.e_phnum; ph++)
	{
		if
		(
			(phdr[ph].p_type == PT_LOAD) &&
			(phdr[ph].p_flags & PF_R)
		)
		{

			unsigned char *buf = new unsigned char[phdr[ph].p_filesz];
			lseek(fd, phdr[ph].p_offset, 0);
			read(fd, buf, phdr[ph].p_filesz);
			printf("Read PHDR: %d, off: %x, size: %d\n", ph, phdr[ph].p_offset, phdr[ph].p_filesz);

			for (unsigned i=0; i<phdr[ph].p_filesz; i++)
			{
				uint32_t addr = phdr[ph].p_paddr + i;
				if (addr < 0x80000000)
					continue;

				addr -= 0x80000000;
				pages[addr / PAGESIZE].data[addr % PAGESIZE] = buf[i];
				pages[addr / PAGESIZE].used = 1;
			}

			delete[] buf;
		}
	}
	delete[] phdr;

	return pages;
} // ELFread

