/** -*- c++ -*- **************************************************************
 **
 ** Copyright 2014 Lynx Technik AG (www.lynx-technik.com)
 **
 ** Licensed under the Apache License, Version 2.0 (the "License");
 ** you may not use this file except in compliance with the License.
 ** You may obtain a copy of the License at
 **
 ** http://www.apache.org/licenses/LICENSE-2.0
 **
 ** Unless required by applicable law or agreed to in writing, software
 ** distributed under the License is distributed on an "AS IS" BASIS,
 ** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ** See the License for the specific language governing permissions and
 ** limitations under the License.
 *****************************************************************************/

#include <string.h>
#include "bitvector.h"

static char hex_char[16]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

void bitvector::append(bitvector &add)
{
	for (bitvector::iterator it = add.begin(); it != add.end(); it++)
		push_back((int)*it);
}

int bitvector::push_back_nibble(const char ch)
{
	int val = -1;

	if ( (ch>='0') && (ch<='9') )
		val = ch - '0';

	if ( (ch>='a') && (ch<='f') )
		val = ch - 'a' + 10;

	if ( (ch>='A') && (ch<='F') )
		val = ch - 'A' + 10;

	for (int i=3; i>=0; i--)
		push_back((val >> i) & 1);

	return val;
}

int bitvector::push_back_string(const char *ch)
{
	int ret;

	for (unsigned i=0;i<strlen(ch);i++)
	{
		ret = push_back_nibble(ch[i]);
		if (ret < 0)
			return ret;
	}

	return 0;
}

int bitvector::push_back_value(const uint64_t val, int len)
{
	for (int i=len-1;i>=0;i--)
		push_back((val >> i) & 1);

	return 0;
}

int bitvector::push_back_value(const uint32_t val)
{
	for (int i=31;i>=0;i--)
		push_back((val >> i) & 1);

	return 0;
}

int bitvector::push_back_buffer(const unsigned size, const unsigned char *poi)
{
	for (unsigned i=size; i>=0; i--)
		push_back((poi[i/8] >> (i%8)) & 1);

	return 0;
}

string bitvector::toString(int format)
{
	string tmp;

	if (format == BINARY)
		for (bitvector::reverse_iterator it = rbegin(); it != rend(); it++)
			tmp += *it ? "1" : "0";

	if (format == HEXADECIMAL)
	{
		int val   = 0;
		int shift = 0;
		for (bitvector::iterator it = begin(); it != end(); it++)
		{
			val |= *it << shift;
			shift++;
			if (shift >= 4)
			{
				tmp = hex_char[val] + tmp;
				val = 0;
				shift = 0;
			}
		}
		if (shift)
			tmp = hex_char[val] + tmp;
	}

	return tmp;
}

uint32_t bitvector::toNum()
{
	uint32_t val   = 0;
	int shift = 0;

	for (bitvector::iterator it = begin(); it != end(); it++)
	{
		val |= *it << shift++;
		if (shift==32)
			break;
	}

	return val;
}
