/** -*- c++ -*- **************************************************************
 **
 ** Copyright 2014 Lynx Technik AG (www.lynx-technik.com)
 **
 ** Licensed under the Apache License, Version 2.0 (the "License");
 ** you may not use this file except in compliance with the License.
 ** You may obtain a copy of the License at
 **
 ** http://www.apache.org/licenses/LICENSE-2.0
 **
 ** Unless required by applicable law or agreed to in writing, software
 ** distributed under the License is distributed on an "AS IS" BASIS,
 ** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ** See the License for the specific language governing permissions and
 ** limitations under the License.
 *****************************************************************************/

#include "bitvector.h"
#include <string>
using namespace std;

#ifndef JTAG_ADAPTER_H
#define JTAG_ADAPTER_H

struct jtagDaisyInfo
{
    int devBefore;
    int devAfter;
    int instBefore;
    int instAfter;
};
typedef struct jtagDaisyInfo jtagDaisyInfo_t;

struct shiftData
{
public:
    unsigned length;
    bitvector tdi;
    bitvector smask;
    bitvector tdo;
    bitvector mask;
};

class jtagAdapter
{
protected:
    int curState;
    struct jtagDaisyInfo chain;

    int checkReadbackData (unsigned len, bitvector *tdo, bitvector *readback, bitvector *mask);

public:
    jtagAdapter();
    virtual ~jtagAdapter() {}
    virtual void setChain(struct jtagDaisyInfo chain);
    int          getState() { return curState; }
    virtual void setJTAGSpeed(int Hz) {}
    virtual void doState(int val) = 0;
    virtual int  shift (
        int shiftState, struct shiftData *dat, int endState,
        bitvector *readback
        ) = 0;
    virtual void runtest(int val, float minTime, float maxTime);
    virtual void clock(int val) = 0;

    virtual int  init() = 0;

    enum STATE {
        RESET     =  0,
        IDLE      =  1,
        DRSELECT  =  2,
        DRCAPTURE =  3,
        DRSHIFT   =  4,
        DREXIT1   =  5,
        DRPAUSE   =  6,
        DREXIT2   =  7,
        DRUPDATE  =  8,
        IRSELECT  =  9,
        IRCAPTURE = 10,
        IRSHIFT   = 11,
        IREXIT1   = 12,
        IRPAUSE   = 13,
        IREXIT2   = 14,
        IRUPDATE  = 15,
        FAIL      = -3,
        OFF       = -2,
        INIT      = -1
    };
};

#endif // JTAG_ADAPTER_H
