/** -*- c++ -*- **************************************************************
 **
 ** Copyright 2014 Lynx Technik AG (www.lynx-technik.com)
 **
 ** Licensed under the Apache License, Version 2.0 (the "License");
 ** you may not use this file except in compliance with the License.
 ** You may obtain a copy of the License at
 **
 ** http://www.apache.org/licenses/LICENSE-2.0
 **
 ** Unless required by applicable law or agreed to in writing, software
 ** distributed under the License is distributed on an "AS IS" BASIS,
 ** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ** See the License for the specific language governing permissions and
 ** limitations under the License.
 *****************************************************************************/

#include <stdio.h>
#include "parallelPort.h"

#if defined(__WIN32__) || defined(__CYGWIN__)

#include <windows.h>
typedef void (WINAPI *PORTOUT) (short int Port, char Data);
typedef char (WINAPI *PORTIN) (short int Port);
HMODULE hio = NULL;
PORTOUT PortOut = NULL;
PORTIN PortIn = NULL;

int parallelPort_usage_counter = 0;
unsigned parallelPort_addresses[] = {0x378, 0x278};

parallelPort::parallelPort(unsigned port)
{
	if ( port > (sizeof(parallelPort_addresses)/sizeof(*parallelPort_addresses)) )
		return;

	// first caller activates driver
	if (!hio)
	{
		hio = LoadLibrary("io.dll");
		if (!hio)
			throw strdup("Loading of library \"io.dll\" failed. Parallel port not available!");

		PortIn = (PORTIN)GetProcAddress(hio, "PortIn");
		PortOut = (PORTOUT)GetProcAddress(hio, "PortOut");

		if ( (!PortIn) || (!PortOut) )
			throw strdup("Internal error in \"io.dll\". Parallel port not available!");
	}

	parallelPort_usage_counter++;
	this->port = port;
}

parallelPort::~parallelPort()
{
	parallelPort_usage_counter--;

	// clean up, we are the last user
	if (!parallelPort_usage_counter)
	{
		FreeLibrary(hio);
		hio = NULL;
	}
}

unsigned char parallelPort::status_in()
{
	return PortIn(parallelPort_addresses[port] + 1);
}

void parallelPort::data_out(unsigned char val)
{
	PortOut(parallelPort_addresses[port], val);
}
#endif

#ifdef linux
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <linux/ppdev.h>

parallelPort::parallelPort(unsigned port)
{
	char dev[] = "/dev/parportx";
	dev[sizeof(dev)-2] = '0' + port;

	this->port = -1;

	int fd = open(dev, O_RDWR);

	if (fd < 0)
	{
		char err[] = "Opening of /dev/parportx failed";
		err[23] = '0' + port;
		throw strdup(err);
	}

	if (ioctl (fd, PPCLAIM, 0))
	{
		close (fd);
		fd = -1;
		throw strdup("Couldn't claim device /dev/parportx");
	}

	this->port = fd;
}

parallelPort::~parallelPort()
{
	if (port >= 0)
	{
		// release the device from our clutches
		ioctl (port, PPRELEASE, 0);

		// and close the handle
		close(port);
	}
}

unsigned char parallelPort::status_in()
{
	unsigned char val = 0;
	ioctl (port, PPRSTATUS, &val);
	return val;
}

void parallelPort::data_out(unsigned char val)
{
	ioctl (port, PPWDATA, &val);
}

#endif

